/***************************************************************************************************************************************************************
 *
 * RenderConsole unit tests
 *
 **************************************************************************************************************************************************************/


const { RenderConsole } = require('../../src/RenderConsole.js');
const StripColor = require('./_common.js').StripColor;


test(`RenderConsole - Should output default text`, () => {
	const test = RenderConsole( 'test', {
		align: 'left',
		colors: [],
		letterSpacing: 0,
		lineHeight: 0,
	}, { width: 100, height: 10 } );

	expect( test.output ).toEqual( ['test'] );
	expect( test.lines ).toEqual( 1 );
});


test(`RenderConsole - Should work with default Size`, () => {
	const test = RenderConsole( 'x', {
		align: 'left',
		colors: [],
		letterSpacing: 0,
		lineHeight: 0,
	});

	expect( test.output ).toEqual( ['x'] );
	expect( test.lines ).toEqual( 1 );
});


test(`RenderConsole - Should output multi-line text`, () => {
	const test = RenderConsole( 'test|test', {
		align: 'left',
		colors: [],
		letterSpacing: 0,
		lineHeight: 0,
	}, { width: 100, height: 10 } );

	expect( test.output ).toEqual( ['test', 'test'] );
	expect( test.lines ).toEqual( 2 );
});


test(`RenderConsole - Should output long text`, () => {
	const test = RenderConsole( 'this is a very long line to test multi lines', {
		align: 'left',
		colors: [],
		letterSpacing: 0,
		lineHeight: 0,
	}, { width: 10, height: 10 } );

	expect( test.output ).toEqual( ['this is a ', 'very long ', 'line to te', 'st multi l', 'ines'] );
	expect( test.lines ).toEqual( 5 );
});


test(`RenderConsole - Should center align text`, () => {
	const test = RenderConsole( 'center', {
		align: 'center',
		colors: [],
		letterSpacing: 0,
		lineHeight: 0,
	}, { width: 10, height: 10 } );

	expect( test.output ).toEqual( ['  center'] );
	expect( test.lines ).toEqual( 1 );
});


test(`RenderConsole - Should not center align text when in browser environment`, () => {
	const test = RenderConsole( 'center', {
		align: 'center',
		colors: [],
		letterSpacing: 0,
		lineHeight: 0,
		env: 'browser',
	}, { width: 10, height: 10 } );

	expect( test.output ).toEqual( ['center'] );
	expect( test.lines ).toEqual( 1 );
});


test(`RenderConsole - Should right align text`, () => {
	const test = RenderConsole( 'right', {
		align: 'right',
		colors: [],
		letterSpacing: 0,
		lineHeight: 0,
	}, { width: 10, height: 10 } );

	expect( test.output ).toEqual( ['     right'] );
	expect( test.lines ).toEqual( 1 );
});


test(`RenderConsole - Should insert letter spacing`, () => {
	const test1 = RenderConsole( 'text', {
		align: 'left',
		colors: [],
		letterSpacing: 2,
		lineHeight: 0,
	}, { width: 10, height: 10 } );

	expect( test1.output ).toEqual( ['t  e  x  t'] );
	expect( test1.lines ).toEqual( 1 );

	const test2 = RenderConsole( 'text', {
		align: 'left',
		colors: [],
		letterSpacing: 3,
		lineHeight: 0,
	}, { width: 100, height: 10 } );

	expect( test2.output ).toEqual( ['t   e   x   t'] );
	expect( test2.lines ).toEqual( 1 );

	const test3 = RenderConsole( 'text', {
		align: 'left',
		colors: [],
		letterSpacing: 10,
		lineHeight: 0,
	}, { width: 100, height: 10 } );

	expect( test3.output ).toEqual( ['t          e          x          t'] );
	expect( test3.lines ).toEqual( 1 );
});


test(`RenderConsole - Should insert letter spacing with multi lines`, () => {
	const test1 = RenderConsole( 'text|text', {
		align: 'left',
		colors: [],
		letterSpacing: 2,
		lineHeight: 0,
	}, { width: 10, height: 10 } );

	expect( test1.output ).toEqual( ['t  e  x  t', 't  e  x  t'] );
	expect( test1.lines ).toEqual( 2 );

	const test2 = RenderConsole( 'text|text', {
		align: 'left',
		colors: [],
		letterSpacing: 3,
		lineHeight: 0,
	}, { width: 100, height: 10 } );

	expect( test2.output ).toEqual( ['t   e   x   t', 't   e   x   t'] );
	expect( test2.lines ).toEqual( 2 );

	const test3 = RenderConsole( 'text|text', {
		align: 'left',
		colors: [],
		letterSpacing: 10,
		lineHeight: 0,
	}, { width: 100, height: 10 } );

	expect( test3.output ).toEqual( ['t          e          x          t', 't          e          x          t'] );
	expect( test3.lines ).toEqual( 2 );
});


test(`RenderConsole - Should insert line height`, () => {
	const test1 = RenderConsole( 'text', {
		align: 'left',
		colors: [],
		letterSpacing: 0,
		lineHeight: 1,
	}, { width: 100, height: 10 } );

	expect( test1.output ).toEqual( ['text'] );
	expect( test1.lines ).toEqual( 1 );

	const test2 = RenderConsole( 'text|text', {
		align: 'left',
		colors: [],
		letterSpacing: 0,
		lineHeight: 3,
	}, { width: 100, height: 10 } );

	expect( test2.output ).toEqual( ['text', '', '', '','text'] );
	expect( test2.lines ).toEqual( 2 );

	const test3 = RenderConsole( 'text|text', {
		align: 'left',
		colors: [],
		letterSpacing: 0,
		lineHeight: 10,
	}, { width: 100, height: 10 } );

	expect( test3.output ).toEqual( ['text', '', '', '', '', '', '', '', '', '', '', 'text'] );
	expect( test3.lines ).toEqual( 2 );
});


test(`RenderConsole - Should insert line height between lines`, () => {
	const test1 = RenderConsole( 'text|text|text', {
		align: 'left',
		colors: [],
		letterSpacing: 0,
		lineHeight: 2,
	}, { width: 100, height: 10 } );

	expect( test1.output ).toEqual( ['text', '', '', 'text', '', '', 'text'] );
	expect( test1.lines ).toEqual( 3 );

	const test2 = RenderConsole( 'text|text|text', {
		align: 'left',
		colors: [],
		letterSpacing: 0,
		lineHeight: 3,
	}, { width: 100, height: 10 } );

	expect( test2.output ).toEqual( ['text', '', '', '', 'text', '', '', '', 'text'] );
	expect( test2.lines ).toEqual( 3 );

	const test3 = RenderConsole( 'text|text|text', {
		align: 'left',
		colors: [],
		letterSpacing: 0,
		lineHeight: 10,
	}, { width: 100, height: 10 } );

	expect( test3.output ).toEqual([
		'text',
		'', '', '', '', '', '', '', '', '', '',
		'text',
		'', '', '', '', '', '', '', '', '', '',
		'text',
	]);
	expect( test3.lines ).toEqual( 3 );
});


test(`RenderConsole - Should insert line height for long lines`, () => {
	const test1 = RenderConsole( 'this is a long line', {
		align: 'left',
		colors: [],
		letterSpacing: 0,
		lineHeight: 2,
	}, { width: 10, height: 10 } );

	expect( test1.output ).toEqual( ['this is a ', '', '', 'long line'] );
	expect( test1.lines ).toEqual( 2 );

	const test2 = RenderConsole( 'this is a long line', {
		align: 'left',
		colors: [],
		letterSpacing: 0,
		lineHeight: 3,
	}, { width: 10, height: 10 } );

	expect( test2.output ).toEqual( ['this is a ', '', '', '', 'long line'] );
	expect( test2.lines ).toEqual( 2 );

	const test3 = RenderConsole( 'this is a long line', {
		align: 'left',
		colors: [],
		letterSpacing: 0,
		lineHeight: 10,
	}, { width: 10, height: 10 } );

	expect( test3.output ).toEqual([
		'this is a ',
		'', '', '', '', '', '', '', '', '', '',
		'long line',
	]);
	expect( test3.lines ).toEqual( 2 );
});


test(`RenderConsole - Should output colored text`, () => {
	const test = RenderConsole( 'test', {
		align: 'left',
		colors: ['red'],
		letterSpacing: 0,
		lineHeight: 0,
	}, { width: 100, height: 10 } );

	expect( test.output ).toEqual( ['\u001b[38;2;255;0;0mtest\u001b[39m'] );
	expect( test.lines ).toBe( 1 );
});


test(`RenderConsole - Should output candy colored text`, () => {
	const test = RenderConsole( 'test', {
		align: 'left',
		colors: ['candy'],
		letterSpacing: 0,
		lineHeight: 0,
	}, { width: 100, height: 10 } );

	expect( StripColor( test.output[ 0 ] ) ).toEqual( 'test' );
	expect( require('util').inspect( test.output[ 0 ] ).length > 'test'.length ).toEqual( true );
	expect( test.lines ).toEqual( 1 );
});


test(`RenderConsole - Should respect small window size`, () => {
	const test = RenderConsole( 'testing long test', {
		align: 'left',
		colors: [],
		letterSpacing: 0,
		lineHeight: 0,
		maxLength: 8,
	}, { width: 7, height: 10 } );

	expect( test.output ).toEqual([ 'testing', ' long t', 'est' ]);
	expect( test.lines ).toEqual( 3 );
});


test(`RenderConsole - Should respect small maxLength over window size`, () => {
	const test = RenderConsole( 'testing long test', {
		align: 'left',
		colors: [],
		letterSpacing: 0,
		lineHeight: 0,
		maxLength: 5,
	}, { width: 7, height: 10 } );

	expect( test.output ).toEqual([ 'testi', 'ng lo', 'ng te', 'st' ]);
	expect( test.lines ).toEqual( 4 );
});
